package de.hdm_stuttgart.mi.javastreams;

import static org.hamcrest.MatcherAssert.assertThat;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.hamcrest.Matchers;
import org.junit.Assert;
import org.junit.Test;

import com.google.common.collect.ImmutableList;
import com.google.common.collect.ImmutableMap;

import de.hdm_stuttgart.mi.javastreams.Student.Sex;

/**
 * Testing functional queries.
 */
public class Java8FunctionalTest {

   final List<Student> roster = Student.createRoster();

   /**
    * Order all male students by email and create a List<String> of their respective
    * names in that order eliminating possible duplicates:
    * 
    *  "Bob", 2, Student.Sex.MALE, "bob@uk.edu"
    *  "Fred", 2, Student.Sex.MALE, "fred@example.com"
    *  "George", 4, Student.Sex.MALE, "george@math.edu"
    *  "Jane", 1, Student.Sex.FEMALE, "jane@kiv.de"
    *  "Kim", 2, Student.Sex.FEMALE, "wilde@serious.de"
    *  
    *  ==> {"Bob", "Fred", "George"}
    *  
    */
   @Test
   public void allMaleDistinctNameOrderedByEmail() {

      final List<String> emails =
            roster.
            stream().
            filter(s -> s.gender == Sex.MALE).
            sorted((s1, s2) -> s1.getEmailAddress().compareTo(s2.getEmailAddress())).
            map(Student::getName).
            distinct().
            collect(Collectors.toList());

      assertThat(
            emails,
            Matchers.<List<String>> equalTo( 
                  ImmutableList.of("Bob", "Fred", "George") 
                  ) 
            );
   }

   /**
    * Summing up all students' marks having a German email address:
    * 
    * "Jane", 1, Student.Sex.FEMALE, "jane@kiv.de"
    * "Kim",  2, Student.Sex.FEMALE, "wilde@serious.de"
    * 
    * Expected value: 1 + 2 == 3
    * 
    * Implementation hint: Map objects to int and provide a suitable reduce operation.
    * You may want to read
    * https://docs.oracle.com/javase/tutorial/collections/streams/reduction.html#reduce
    * 
    */
   @Test
   public void markSumAllStudentsMarksHavingEmail_Dot_de() {
      Assert.fail("Implement me!");// TODO
   }

   /**
    * Get the average mark of all female students as double value.
    * 
    * Implementation hint: Map objects to int and provide a suitable reduce operation.
    * You may want to read
    * https://docs.oracle.com/javase/8/docs/api/java/util/stream/IntStream.html#average--
    * 
    */
   @Test
   public void averageMarkFemaleStudents() {
      Assert.fail("Implement me!");// TODO
   }

   /**
    * Create a Map<Student.Sex, List<String>> grouping students by sex and collecting
    * their respective names:
    * 
    * Female: 
    *     "Jane"
    *     "Kim"
    *
    * Male:
    *     "Fred"
    *     "George"
    *     "Bob"
    * 
    * Implementation hint:
    * stackoverflow.com/questions/2509293/map-equality-using-hamcrest#answer-29680356
    * 
    */
   @Test
   public void studentNamesBySex() {

      final Map<Student.Sex, List<String>> studentnamesBySex = null; // TODO

      assertThat(studentnamesBySex,
            Matchers.<Map<Student.Sex, List<String>>> equalTo( 
                  ImmutableMap.of (
                        Student.Sex.MALE,   ImmutableList.of("Fred", "George", "Bob")
                        ,Student.Sex.FEMALE, ImmutableList.of("Jane", "Kim")
                        ) 
                  ) 
            );
   }

   /**
    * Create a Map<Integer, Integer> of student's marks among with their corresponding frequencies:
    * 
    * Mark 1: "Jane"               --- 1 student
    * Mark 2: "Fred", "Bob", "Kim" --> 3 students
    * Mark 4: "George"             --> 1 student
    * 
    */
   @Test
   public void markingFrequencies() {
      Assert.fail("Implement me!");// TODO
   }

   /**
    * Students' names in ascending order grouped by by marking category:
    * 
    * Mark 1: {"Jane"}
    * Mark 2: {"Bob", "Fred", "Kim"}
    * Mark 4: {"George"} 
    * 
    * Implementation hint: Sort the stream by student's names beforehand. Then
    * apply a suitable 
    * {@link Collectors#mapping(java.util.function.Function, java.util.stream.Collector)}
    * 
    */
   @Test
   public void studentsByMark() {
      Assert.fail("Implement me!");// TODO
   }
}
