package dom.xpath;

import java.io.IOException;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;

import javax.ws.rs.core.Response;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;


/** Check Readability of a given URL
 * @author $Author: goik $
 * @version $Rev: 290 $ $Date: 2007-11-19 17:49:15 +0100 (Mo, 19. Nov 2007) $
 *
 * $URL: https://version.mi.hdm-stuttgart.de/svn/goikvorles/trunk/Vorlesungen/WEB-INF/src/dom/xpath/CheckUrl.java $.
 *
 */
public class CheckUrl {
   
   static private final Logger log = LogManager.getLogger(CheckUrl.class);
   
  /** Check whether a given object like http://someserver.com/img.gif
   *  can be retrieved.
   *  
   * @param urlRef
   *  The URL to be checked for read access
   *  
   * @return An informative URL related status message or null e.g. if the protocol in question is unknown.
   */
  public static Response.Status checkReadability(final String urlRef) {
    try {
      final URL url = new URL(urlRef);
      
      switch(url.getProtocol()) {
      
      case "http":
      case "https":
         try {
            final HttpURLConnection huc =  (HttpURLConnection) url.openConnection();
            huc.setRequestMethod("GET"); 
            huc.connect(); 
            return Response.Status.fromStatusCode(huc.getResponseCode());            
         } catch (final IOException e) {
            log.error("Unable to connect to " + urlRef, e);
         }
         break;        
         default:
            log.error("Protocol '" + url.getProtocol() + "' not yet implemented");
      }
      
    } catch (MalformedURLException e) {
      System.err.println("Adress '" + urlRef + "' is malformed");
    }
    return null;
  }
}